/*
 * Example code how to use Optimize/Dfmpr
 *
 * Copyright (c) 2002-2013,
 *   Arno Formella (formella@ei.uvigo.es)
 *   Ubaldo Garcia-Palomares (garciap@usb.ve)
 *   Jose F. Rodriguez (jrodri@usb.ve)
 */

/*
 * Academic Free License v2.0
 * 
 * http://www.opensource.org/licenses/
 */

#include <cmath>
#include <iostream>

// we need the *typical* vector space operations
// we define a class of a 1D vector that provides the necessary operations

class Vector {
private:
  double x;
public:
  Vector() : x(0.0) { }
  Vector(double x_) : x(x_) { }
  Vector(const Vector& V) : x(V.x) { }
  Vector& operator=(const Vector& V) { x=V.x; }
  Vector operator+(const Vector& V) const { return Vector(x+V.x); }
  Vector operator-(void) const { return Vector(-x); }
  Vector operator-(const Vector& V) const { return Vector(x-V.x); }
  Vector operator*(double a) { return Vector(a*x); }
  double& operator[](int i) { return x; }
  double operator[](int i) const { return x; }
  double norm(void) const { return x; }
  friend Vector operator*(double a, const Vector& V) { return Vector(a*V.x); }
  friend std::ostream& operator<<(std::ostream& O, const Vector& V) {
    O << V.x;
  }
};

#include "dfmpr.h"

// derive out problem from the abstract OptimizationProblem class
class Problem : public mcl::OptimizationProblem<Vector> {
public:
  // our dimension is 1 in this simple example
  virtual unsigned int Dimension(void) const { return 1; }
  // THE objective function is just  x*x
  virtual double Fobj(
    const Vector& V
  ) {
    return V[0]*V[0];
  }
};

int main(
) {
  Problem problem;
  mcl::Dfmpr<Vector> O(problem,10000,10000,1E-10);
  Vector v(1.0);
  O.SetRandomSeed();
  O.Run(v);
  std::cout << "xmin: " << v << std::endl;
  std::cout << "fmin: " << O.MinimumFobj() << std::endl;
  std::cout << "iter: " << O.Iterations() << std::endl;
  std::cout << "eval: " << O.Evaluations() << std::endl;
}

